local Stats = require "engine.interface.ActorStats"
local Particles = require "engine.Particles"
local Shader = require "engine.Shader"
local Entity = require "engine.Entity"
local Chat = require "engine.Chat"
local Map = require "engine.Map"
local Level = require "engine.Level"
local Talents = require "engine.interface.ActorTalents"

newEffect{
	name = "K_NALOREN_SPIRIT", image = "talents/k_naloren_spirit.png",
	desc = "Naloren Spirit",
	long_desc = function(self, eff) return ("The target is filled with the Spirit of the Nalore, only dying when reaching - %d life and increasing Spell, Mind,%s and Physical Power by %d ."):format(eff.dieat, Talents.talents_def.T_STEAM_POOL and " Steam," or "",  eff.power) end,
	type = "physical",
	subtype = {  },
	status = "beneficial",
	parameters = { dieat = 10, power = 10 },
	on_gain = function(self, err) return "#Target# is filled with the Spirit of the Nalore!", "+ Spirit!" end,
	on_lose = function(self, err) return "#Target# is filled with the Spirit of the Nalore no longer.", "- Spirit" end,
	activate = function(self, eff)
		self:effectTemporaryValue(eff, "die_at", -eff.dieat)
		self:effectTemporaryValue(eff,"combat_dam", eff.power)
		self:effectTemporaryValue(eff, "combat_mindpower", eff.power)
		self:effectTemporaryValue(eff, "combat_spellpower", eff.power)
		if Talents.talents_def.T_STEAM_POOL then
			self:effectTemporaryValue(eff, "combat_steampower", eff.power)
		end
	end,
	deactivate = function(self, eff)
		self:removeTemporaryValue(eff, "die_at", -eff.dieat)
		self:removeTemporaryValue(eff,"combat_dam", eff.power)
		self:removeTemporaryValue(eff, "combat_mindpower", eff.power)
		self:removeTemporaryValue(eff, "combat_spellpower", eff.power)
		if Talents.talents_def.T_STEAM_POOL then
			self:removeTemporaryValue(eff, "combat_steampower", eff.power)
		end
	end,
	
}

--passively gain crit shrug and defense based on Constitution, at 1/2 the rate of Dexterity. On cooldown, when you are hit with a negative status effect, you will remove a detrimental effect each time you move, for one turn.
newEffect{
	name = "K_MERCURIAL", image = "talents/k_mercurial.png",
	desc = "Mercurial",
	long_desc = function(self, eff) return ("The target is very mercurial, increasing movement speed by %d%% and gaining a %d%% chance to shed one detrimental effect each time it moves."):format(eff.speed*100, eff.chance) end,
	type = "physical",
	subtype = {  },
	status = "beneficial",
	parameters = { chance = 50, speed= 0.45},
	on_gain = function(self, err) return "#Target# is more mercurial!", "+ Mercurial!" end,
	on_lose = function(self, err) return "#Target# is mercurial no longer.", "- Mercurial" end,
	callbackOnMove = function(self, eff, moved, force, ox, oy, x, y)
		if moved and not (self.x == ox and self.y == oy) then
			if rng.percent(eff.chance) then
				self:removeEffectsFilter({status="detrimental", ignore_crosstier=false}, 1)
			end
		end
	end,
	activate = function(self, eff)
		self:effectTemporaryValue(eff,"movement_speed", eff.speed)
	end,
}

newEffect{
	name = "K_FLUID_MOTION", image = "talents/k_whitewater.png",
	desc = "Fluid Motion",
	long_desc = function(self, eff) return ("The target is able to move with the current, gaining 150%% movement speed."):format() end,
	type = "physical",
	subtype = {  },
	status = "beneficial",
	parameters = {},
	activate = function(self, eff)
		self:effectTemporaryValue(eff, "movement_speed", 1.5)
	end,
}

newEffect{
	name = "K_FLOUNDER", image = "talents/k_whitewater.png",
	desc = "Floundering",
	long_desc = function(self, eff) return ("The target is floundering in the torrent. Each time they act they will have a 50%% chance to be pushed one tile in a random direction by the raging water. If the target is pushed they will become Unbalanced for two turns afterward, causing talent use to have a 75%% chance to fail.") end,
	type = "physical",
	subtype = {  },
	status = "detrimental",
	parameters = {chance = 50},
	on_gain = function(self, err) return "#Target# is floundering!", "+Floundering!" end,
	on_lose = function(self, err) return "#Target# is no longer floundering.", "-Floundering" end,
	callbackOnAct = function(self, eff)
		if rng.percent(eff.chance) then
			local ox = self.x
			local oy = self.y
			local tiles = {}
			local grids = core.fov.circle_grids(ox, oy, 1, true)
			for x, yy in pairs(grids) do 
				for y, _ in pairs(grids[x]) do
					local is_blocked = game.level.map:checkEntity(x, y, game.level.map.TERRAIN, "block_move")
					local actor = game.level.map(x, y, Map.ACTOR)
					if not is_blocked then
						if not actor then
							tiles[#tiles+1] = {x=x , y=y}
						end
					end
				end 
			end
			if #tiles >= 1 then
				local p = rng.tableRemove(tiles)
				self:move(p.x, p.y, true)
				if config.settings.tome.smooth_move > 0 then
					self:resetMoveAnim()
					self:setMoveAnim(ox, oy, 8, 5)
				end
				self:setEffect(self.EFF_K_UNBALANCED, 2, {})
			end
		end
	end,
	activate = function(self, eff)
	end,
}

newEffect{
	name = "K_UNBALANCED", image = "talents/k_unbalanced.png",
	desc = "Unbalanced",
	long_desc = function(self, eff) return ("The target has been unbalanced by the raging waters, casuing all talent use to fail."):format() end,
	type = "physical",
	subtype = {  },
	status = "detrimental",
	parameters = {},
	activate = function(self, eff)
		eff.tmpid = self:addTemporaryValue("talent_fail_chance", 75)
	end,
	deactivate = function(self, eff)
		self:removeTemporaryValue("talent_fail_chance", eff.tmpid)
	end,
}