-- ToME - Tales of Maj'Eyal
-- Copyright (C) 2009 - 2015 Nicolas Casalini
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Nicolas Casalini "DarkGod"
-- darkgod@te4.org

-- Generic requires for racial based on talent level
racial_req1 = {
	level = function(level) return 0 + (level-1)  end,
}
racial_req2 = {
	level = function(level) return 8 + (level-1)  end,
}
racial_req3 = {
	level = function(level) return 16 + (level-1)  end,
}
racial_req4 = {
	level = function(level) return 24 + (level-1)  end,
}


newTalentType{ type="race/naloren", name = "Naloren", generic = true, description = "The various racial bonuses a character can have."}

doWieldCheck = function(self, t, type)
	local wield = false
	local weapon = false
	local type2h = false
	if type == "mace" then
		type2h = "greatmaul"
	elseif type == "waraxe" then
		type2h = "battleaxe"
	elseif type == "longsword" then
		type2h = "greatsword"
	elseif type == "trident" then
		type2h = "trident"
	end
	if self:getInven("MAINHAND")[1] then
		weapon = self:getInven("MAINHAND")[1]
	end
	local offweapon = false
	if self:getInven("OFFHAND")[1] then
		offweapon = self:getInven("OFFHAND")[1]
	end
	if weapon then
		if weapon.subtype == type then
			wield = true
		elseif weapon.subtype == type2h then
			wield = true
		end
	end
	if offweapon then
		if offweapon.subtype == type then
			wield = true
		end
	end
	return wield
end

newTalent{
	short_name = 'K_NALOREN_SPIRIT',
	name = "Spirit of the Tides",
	type = {"race/naloren", 1},
	require = racial_req1,
	points = 5,
	no_energy = true,
	cooldown = function(self, t) return math.ceil(self:combatTalentLimit(t, 8, 45, 25)) end, 
	tactical = { ATTACK = 1, DEFEND = 1 },
	getDieAt = function(self, t)
		return 25 + self:combatStatScale("con", 1, 2)*self.max_life/5
	end,
	getPower = function(self, t)
		return self:combatStatScale(math.max(self:getDex(), self:getCon()), 20, 80)
	end,
	-- passives = function(self, t, p)
		-- self:talentTemporaryValue(p, "can_breath", {water = 1})
	-- end,
	action = function(self, t)
		self:setEffect(self.EFF_K_NALOREN_SPIRIT, 6, {power=t.getPower(self,t), dieat=t.getDieAt(self,t)})
		return true
	end,
	info = function(self, t)
		local s =""
		if Talents.talents_def.T_STEAM_POOL then
			s = " Steam,"
		end
		local power = t.getPower(self, t)
		local dieat = t.getDieAt(self, t)
		return ([[You call upon the surging spirit of the seas to gain a boost in combat. For 6 turns you will only die when reaching - %d life and your raw Spell, Mind,%s and Physical Power will all increase by %d . 
			Death limit will scale with your Constitution and Power bonuses scales with the higher of your Dexterity and Constitution.
     
	 Passively, you have mastered the techniques of your elders', allowing you to easily breath underwater.]]):format(dieat, s, power)
	end,
}

newTalent{
	short_name = 'K_MAIL_MASTERY',
	name = "Naloren Combat Style",
	type = {"race/naloren", 2},
	mode = "passive",
	require = racial_req2,
	points = 5,
	no_unlearn_last = true,
	getSpeed = function(self, t) return self:combatTalentLimit(t, 0.15, 0.035, 0.10)/2 end,
	getDef = function(self, t) return self:combatTalentLimit(t, 25, 5, 20)/2 end,
	getSpeedBonus = function(self, t) return self:combatStatLimit("dex", 2.2, 1, 2) end,
	getDefBonus = function(self, t) return self:combatStatLimit("con", 2.2, 1, 2) end,
	on_learn = function(self, t)
		if self:getTalentLevelRaw(t) == 1 then
			if self.player then
				self.__show_special_talents = self.__show_special_talents or {}
				self.__show_special_talents[self.T_EXOTIC_WEAPONS_MASTERY] = true
			end
			--self:learnTalent(self.T_ARMOUR_TRAINING, true, nil, {no_unlearn=true})
			self:attr("max_encumber", 15)
			self:attr("fatigue", -2.5)
		else
			self:attr("max_encumber", 15)
			self:attr("fatigue", -2.5)
		end
	end,
	on_unlearn = function(self, t)
		if not self:knowTalent(self.T_K_MAIL_MASTERY) then
			--self:unlearnTalent(self.T_ARMOUR_TRAINING) 
			self:attr("max_encumber", -15)
			self:attr("fatigue", 2.5)
			if self.temp_nal_armor then
				self:removeTemporaryValue("cancel_damage_chance", self.temp_nDef)
				self:removeTemporaryValue("combat_physspeed", self.temp_nAspeed)
				self:removeTemporaryValue("movement_speed", self.temp_nMspeed)
				self.temp_nal_armor = nil
			end
		else
			self:attr("max_encumber", -15)
			self:attr("fatigue", 2.5)
		end
	end,
	callbackOnWear = function(self, t, o)
		if o.type == "weapon" then
			if o.subtype == "trident"  then
				self:learnTalent(self.T_K_THRUST, true, 1, {no_unlearn=true})
			elseif o.subtype == "whip"  then
				self:learnTalent(self.T_K_LASH, true, 1, {no_unlearn=true})
			end
		end
	end,
	callbackOnTakeoff = function(self, t, o)
		if o.type == "weapon" then
			if o.subtype == "trident" and self:knowTalent(self.T_K_THRUST) then
				self:unlearnTalent(self.T_K_THRUST)
			elseif o.subtype == "whip" and self:knowTalent(self.T_K_LASH) then
				if not doWieldCheck(self, t, "whip") then
					self:unlearnTalent(self.T_K_LASH)
				end
			end
		end
	end,
	callbackOnAct = function(self, t)
	--remove old values
		if self.temp_nal_armor then
			self:removeTemporaryValue("cancel_damage_chance", self.temp_nDef)
			self:removeTemporaryValue("combat_physspeed", self.temp_nAspeed)
			self:removeTemporaryValue("movement_speed", self.temp_nMspeed)
			self.temp_nal_armor = nil
		end
	--make sure we still know the talent and still have Mail armor on
		if self:knowTalent(self.T_K_MAIL_MASTERY) and self:hasHeavyArmor() and not self:hasMassiveArmor() then
	--set new values
		self.temp_nal_armor = true
		self.temp_nAspeed = self:addTemporaryValue("combat_physspeed", t.getSpeed(self, t)*t.getSpeedBonus(self, t))
		self.temp_nMspeed = self:addTemporaryValue("movement_speed", t.getSpeed(self, t)*t.getSpeedBonus(self, t))
		self.temp_nDef = self:addTemporaryValue("cancel_damage_chance", t.getDef(self, t)*t.getDefBonus(self, t))
		end
	end,
	info = function(self, t)
		local spd = t.getSpeed(self, t)*t.getSpeedBonus(self, t)
		local def = t.getDef(self, t)*t.getDefBonus(self, t)
		local fat = self:getTalentLevelRaw(t)*2.5
		local enc = self:getTalentLevelRaw(t)*15
		return ([[Your people are legendary for their use of mail armor and exotic weapons. You have learned special techniques for Whips and Tridents that will become available when equipping them, and you will be able to equip heavy(mail) armor, boots, helms and gauntlets. 
		Learning this talent will also reveal Exotic Weapon Mastery. #GREEN#(Note: you will need to close the level-up dialog to validate the change.)#WHITE#
			
			While wearing Heavy(Mail) armor, your attack speed and movement speed will increase by %0.1f%% based on Dexterity and you will gain a %d%% chance to completely negate damage based on Constitution.
			Your training has toughend you, passively granting + %d maximum encumbrance and reducing Fatigue by %d . 
			Taking the fifth talent point will change the requirements to equip Heavy(Mail) armor and to learn Armour Training from Strength to Constitution.]]):format(spd*100, def, enc, fat)
	end,
}



newTalent{
	short_name = 'K_MERCURIAL',
	name = "Mercurial",
	type = {"race/naloren", 3},
	require = racial_req3,
	points = 5,
	mode = "passive",
	cooldown = 12,
	tactical = { DEFENSE = 2 },
	getSpeed = function(self, t)
		return self:combatTalentStatDamage(t, "dex", 100, 200)/100
	end,
	getChance = function(self, t)
		return self:combatTalentStatDamage(t, "con", 10, 50)
	end,
	on_learn = function(self, t)
		if self:getTalentLevelRaw(t)  == 1 then
			local def = self:getCon()*0.23
			local shrug = self:getCon()*0.20
			self.temp_nal_merc = true
			self.temp_nal_merc_def = self:addTemporaryValue("combat_def", def)
			self.temp_nal_merc_shrug = self:addTemporaryValue("ignore_direct_crits", shrug)
		end
	end,
	on_unlearn = function(self, t)
		if not self:knowTalent(self.T_K_MERCURIAL) then
			if self.temp_nal_merc then
				self:removeTemporaryValue("combat_def", self.temp_nal_merc_def)
				self:removeTemporaryValue("ignore_direct_crits", self.temp_nal_merc_shrug)
				self.temp_nal_merc = nil
			end
		end
	end,
	callbackOnStatChange = function(self, t, stat, v)
		if stat == self.STAT_CON then
			if self.temp_nal_merc then
				local def = self:getCon()*0.23
				local shrug = self:getCon()*0.20
				self:removeTemporaryValue("combat_def", self.temp_nal_merc_def)
				self:removeTemporaryValue("ignore_direct_crits", self.temp_nal_merc_shrug)
				self.temp_nal_merc_shrug = nil
				self.temp_nal_merc_def = nil
				self.temp_nal_merc_shrug = self:addTemporaryValue("ignore_direct_crits", shrug)
				self.temp_nal_merc_def = self:addTemporaryValue("combat_def", def)
			end
		end
	end,
	callbackOnTemporaryEffect = function(self, t, eff_id, e, p)
		if e.type ~= "other" and e.status == "detrimental" and not self:isTalentCoolingDown(t) then
			self:setEffect(self.EFF_K_MERCURIAL, 3, {chance=t.getChance(self, t), speed = t.getSpeed(self, t)})
			self:startTalentCooldown(t)
		end
	end,
	info = function(self, t)
		local def = self:getCon()*0.23
		local shrug = self:getCon()*0.20
		local speed = t.getSpeed(self, t)
		local chance = t.getChance(self, t)
		return ([[A strong body is a flexible body, and your agility increases with your resilience. You will gain %d defense and a %d%% chance to shrug off critical damage, based on Constitution.
			Additionally, when afflicted with a detrimental status effect (except for type 'other'), your body reacts by becoming even more mercurial. For three turns, your movement speed will increase by %d%% and you will have a %d%% chance to shed one detrimental effect each time you move. 
		This effect has a cooldown. Speed will increase with Dexterity and chance to shed effects with Constitution.]]):format(def, shrug, speed*100, chance)
	end,
}


newTalent{
	short_name = 'K_WHITEWATER',
	name = "Whitewater",
	type = {"race/naloren", 4},
	require = racial_req4,
	points = 5,
	cooldown = function(self, t) 
		return math.ceil(self:combatTalentLimit(t, 10, 35, 20)) 
	end,
	--no_energy = true,
	proj_speed = 10,
	tactical = { CLOSEIN = 1, ESCAPE = 1 },
	range = function(self, t)
		return self:combatTalentLimit(t, 8, 3, 6)
	end,
	requires_target = false,
	radius = function(self, t)
		return self:combatTalentLimit(t, 8, 3, 6)
	end,
	target = function(self, t)
		return {type="ball", range=self:getTalentRange(t), radius=self:getTalentRadius(t), selffire=true,display={particle="bolt_ice", particle_args={size_factor=1.5}}}
	end,
	action = function(self, t)
		local tg = self:getTalentTarget(t)
		-- Add a lasting map effect
		local x, y = self:getTarget(tg)
		if not x or not y or not self:canProject(tg, x, y) then return nil end
		self:projectile(tg, x, y, DamageType.K_WHITEWATER, 0, function(self, tg, x, y, grids)
		game.level.map:addEffect(self,
			x, y, 6,
			DamageType.K_WHITEWATER, 0,
			tg.radius,
			5, nil,
			MapEffect.new{color_br=0, color_bg=50, color_bb=250, effect_shader="shader_images/water_effect1.png"},
			nil,
			tg.selffire
		)
		end)
		game:playSoundNear(self, "talents/water")
		return true
	end,
	info = function(self, t)
		local rad = self:getTalentRadius(t)
		return([[You are one with the Tides and can call the seas to aid you. Call a circle of rushing water with radius %d in a range of %d for 6 turns.
		All foes within the area will become Wet and Flounder as they fight the torrent. 
		Floundering targets will have a 50%% chance to be pushed one tile in a random direction each time they act. If the target is pushed, they will become badly unbalanced for two turns afterward, causing all talent use to have a 75%% chance to fail.
		You and your allies are able to move with the currents, gaining the Fluid Motion effect, increasing movement speed by 150%%.
		The Fluid Motion and Flounder effects each last one turn, and will be reapplied each turn the actor remains in the affected area.]]):format(rad, rad)
	end,
	
}

--Whiplash
if not Talents.talents_def.T_K_LASH then -- Glad compatability
newTalent{
	short_name = 'K_LASH',
	name = "Lash",
	type = {"technique/objects", 1},
	points = 1,
	cooldown = 6,
	no_energy = fake,
	speed = "weapon",
	tactical = { ATTACK = 1 },
	no_unlearn_last = true,
	range = 5,
	requires_target = true,
	is_melee = true,
	on_pre_use = function(self, t)
		if not doWieldCheck(self, t, "whip") then return false end
		return true
	end,
	action = function(self, t)
		local has_MainWhip = false
		local has_OffWhip = false
		local weapon = self:getInven("MAINHAND")[1]
		local offweapon = self:getInven("OFFHAND")[1]
		if weapon and weapon.subtype == "whip" then
			has_MainWhip = true
		end
		if offweapon and offweapon.subtype == "whip" then
			has_OffWhip = true
		end
		if not (has_MainWhip or has_OffWhip) then return nil end
		local tg = {type="hit", range=self:getTalentRange(t), talent=t}
		local x, y = self:getTarget(tg)
		if not x or not y then return nil end
		local _ _, x, y = self:canProject(tg, x, y)
		local target = game.level.map(x, y, Map.ACTOR)
		if target then
			if has_MainWhip and has_OffWhip then
				self:attackTarget(target, nil, 1, true)
			elseif has_MainWhip then
				self:attackTargetWith(target, weapon.combat, nil, 1)
			else
				self:attackTargetWith(target, offweapon.combat, nil, self:getOffHandMult(offweapon.combat, 1))
				game:playSoundNear(self, "actions/whip_hit") --because it doesn't make that satisfying whip-crack sound effect when you use an off-hand whip
			end
		else
			return
		end
		return true
	end,
	info = function(self, t)
		return([[Lash out with your whip(s), striking an enemy up to 5 tiles away.]]):format()
	end,
}
end

if not Talents.talents_def.T_K_THRUST then
newTalent{
	short_name = 'K_THRUST',
	name = "Thrust",
	type = {"technique/objects", 1},
	points = 1,
	cooldown = 6,
	no_energy = fake,
	speed = "weapon",
	tactical = { ATTACK = 1 },
	no_unlearn_last = true,
	range = 3,
	requires_target = true,
	is_melee = true,
	on_pre_use = function(self, t)
		if not doWieldCheck(self, t, "trident") then return false end
		return true
	end,
	action = function(self, t)
		local weapon = self:getInven("MAINHAND")[1]
		if not weapon or not weapon.subtype == "trident" then return nil end
		local tg = {type="beam", range=self:getTalentRange(t), talent=t}
		local x, y = self:getTarget(tg)
		if not x or not y then return nil end
		self:project(tg, x, y, function(px, py, tg, self)
			local target = game.level.map(px, py, Map.ACTOR)
			if target then
				self:attackTargetWith(target, weapon.combat, nil, 1)
			end
		end)
		return true
	end,
	info = function(self, t)
		return([[Thrust with your trident, striking up to 3 tiles in a beam.]]):format()
	end,
}
end

--Armour Training requirement change
Talents.talents_def.T_ARMOUR_TRAINING.require = function(self, t)
	if self:knowTalent(self.T_K_MAIL_MASTERY) then
		local t2 = self:getTalentFromId(self.T_K_MAIL_MASTERY)
		local lev = self:getTalentLevelRaw(t2)
		if lev >= 5 then
			return {stat = {con = function(level) return 16 + (level + 2) * (level - 1) end}}
		else
			return {stat = {str = function(level) return 16 + (level + 2) * (level - 1) end}}
		end
	else
		return {stat = {str = function(level) return 16 + (level + 2) * (level - 1) end}}
	end
end