local Talents = require("engine.interface.ActorTalents")

local alter

if not currentZone.is_demonic then
	alter = function(add, mult)
		add = add or 0
		mult = mult or 1
		return function(e)
			if e.rarity then
				local list = {"T_TENTACLE_GRAB", "T_SLIME_SPIT"}
				e[#e+1] = resolvers.talents{[ Talents[rng.table(list)] ] = {base=1, every=5, max=3}}
				e.rarity = math.ceil(e.rarity * mult + add)
				e.faction = "horrors"
				e.name = rng.table{"horrific ", "warped ", "writhing "}..e.name
			end
		end
	end
else
	alter = function(add, mult)
		add = add or 0
		mult = mult or 1
		return function(e)
			if e.rarity then
				local list = {"T_WATER_BOLT", "T_FROST_GRAB"}
				e[#e+1] = resolvers.talents{[ Talents[rng.table(list)] ] = {base=1, every=5, max=3}}
				e.rarity = math.ceil(e.rarity * mult + add)
				e.subtype = "demon"
				e.name = rng.table{"demonic ", "twisted ", "fiendish "}..e.name
			end
		end
	end
end

load("/data/general/npcs/aquatic_critter.lua", alter(0))
if not currentZone.is_demonic then
	load("/data/general/npcs/horror_aquatic.lua", alter(0))
	newEntity{ base = "BASE_NPC_HORROR_AQUATIC",
		name = "minor swarming horror", color=colors.GREY,
		desc ="This tiny fish-like horror moves erratically, but is constantly surrounded by others of its kind.",
		resolvers.nice_tile{image="invis.png", add_mos = {{image="npc/horror_aquatic_swarming_horror.png"}}},
		level_range = {1, nil}, exp_worth = 1,
		rarity = 2,
		hive_swarm_rarity = 1,
		rank=1,
		autolevel = "zerker",
		combat_armor = 0, combat_def = 5,
		max_life = 5, life_rating=-2,
		combat = { dam=8, atk=5, apr=0, dammod={str=0.6}, damtype=DamageType.PHYSICAL},
		ai = "dumb_talented_simple", ai_state = { ai_move="move_complex",},
		
		resolvers.talents{
			[Talents.T_BLINDSIDE]={base=1, every=6, max=3},
			[Talents.T_BLOOD_SUCKERS]={base=5, every=1, max=10},
		},

		make_escort = {
			{type="horror", subtype="aquatic", name="minor swarming horror", number=2, no_subescort=true},
		},
	}
	newEntity{ base = "BASE_NPC_HORROR_AQUATIC",
		name = "small swarm hive", color=colors.BLACK,
		desc ="This mass of flesh pulsates and churns as tiny fish-like horrors burst from its orifices.",
		resolvers.nice_tile{image="invis.png", add_mos = {{image="npc/horror_aquatic_swarm_hive.png"}}},
		level_range = {1, nil}, exp_worth = 1,
		rarity = 3,
		rank = 3,
		size_category = 3,
		autolevel = "wildcaster",
		combat_armor = 16, combat_def = 8,
		hate_regen=1,
		max_life = 40, life_rating = 0, fixed_rating = true,
		combat = {dam=15, apr=0, atk=20, dammod={wil=1}},
		ai = "tactical",

		never_move = 1,

		resists = {all = 10,},
		
		summon = {{type="horror", subtype="aquatic", name="minor swarming horror", number=2, special_rarity="hive_swarm_rarity", hasxp=false}, },

		resolvers.talents{
			[Talents.T_SUMMON]=1,
		
			[Talents.T_WILLFUL_STRIKE]={base=2, every=6, max=8},
			--[Talents.T_MINDLASH]={base=3, every=5, max=9},
			--[Talents.T_BLAST]={base=1, every=7, max=7},
		},	
	}
	newEntity{ base = "BASE_NPC_HORROR_AQUATIC", define_as = "SHRIEKING_ABYSSAL_HORROR",
		name = "Swarming Abyssal Horror", color=colors.BLACK,
		desc = "This pitch black form is shrouded in darkness and swarming with horrors. All you can make out are a pair of deep red eyes, hidden behind a mass of tentacles.",
		resolvers.nice_tile{image="invis.png", add_mos = {{image="npc/horror_aquatic_abyssal_horror.png"}}},
		level_range = {7, nil}, exp_worth = 2,
		instakill_immune = 1,
		rank = 4,
		size_category = 2,
		autolevel = "wildcaster",
		max_life = 80, life_rating = 10, fixed_rating = true,
		hate_regen=4,
		combat_armor = 0, combat_def = 16,
		body = { INVEN = 10, MAINHAND=1, OFFHAND=1, BELT=1, BODY=1 },
		resolvers.equip{
			{type="armor", subtype="belt", defined="SLIPPERY_BELT", random_art_replace={chance=75}},
			{type="weapon", subtype="mindstar", autoreq=true, force_drop=true},
			{type="weapon", subtype="mindstar", autoreq=true, force_drop=true},
		},
		resolvers.drops{chance=100, nb=3, {tome_drops="boss"} },
		combat = {
			dam=15,
			atk=12, apr=4,
			dammod={mag=0.8, wil=0.2}, physcrit = 6,
			damtype=engine.DamageType.DARKNESS,
		},

		ai = "tactical", ai_state = { ai_move="move_complex", talent_in=1, ally_compassion=0 },

		resists = {[DamageType.DARKNESS] = 25, [DamageType.LIGHT] = -25},
		resolvers.auto_equip_filters("Oozemancer"),
		auto_classes={{class="Oozemancer", start_level=12, level_rate=35}},
		summon = {{type="horror", subtype="aquatic", name="minor swarming horror", number=3, special_rarity="hive_swarm_rarity", hasxp=false}, },
		-- Override the recalculated AI tactics to avoid problematic kiting in the early game
		low_level_tactics_override = {escape=0},

		on_die = function(self, who)
			--game.player:resolveSource():setQuestStatus("start-naloren", engine.Quest.COMPLETED, "abyssal-caves")
		end,
		resolvers.talents{
				[Talents.T_SLIME_SPIT]={base=1, every=5, max=8},
				[Talents.T_CREEPING_DARKNESS]={base=1, every=8, max=3},
				[Talents.T_DARK_VISION]=5,
				[Talents.T_SUMMON]=1,
				
				[Talents.T_ABYSSAL_SHROUD]={base=1, every=8, max=3},
				
				[Talents.T_TENTACLE_GRAB]={base=1, every=5, max=3},
		},
		resolvers.sustains_at_birth(),
	}
else
	load("/data/general/npcs/aquatic_demon.lua", alter(0))
		newEntity{ base = "BASE_NPC_AQUATIC_DEMON",
		name = "lesser water imp", color=colors.YELLOW_GREEN,
		display = "u",
		desc = "A tiny water demon, casting annoying spells at you.",
		resolvers.nice_tile{image="invis.png", add_mos = {{image="npc/aquatic_demon_water_imp.png"}}},
		level_range = {1, nil}, exp_worth = 1,
		rarity = 1,
		rank = 2,
		size_category = 1,
		autolevel = "caster",
		combat_armor = 1, combat_def = 0,
		combat = {damtype=DamageType.ICE},

		resists={[DamageType.COLD] = resolvers.mbonus(12, 5)},

		resolvers.talents{ [Talents.T_GLACIAL_VAPOUR]=3, [Talents.T_PHASE_DOOR]=4, },
	}
	newEntity{ base = "BASE_NPC_AQUATIC_DEMON", define_as = "HERALD_OF_WALROG",
		name = "Herald of Walrog", color=colors.DARK_SEA_GREEN, unique=true,
		desc = "The Herald of Walrog, Demon Lord of Water. Walrog destroyed the original Shellsea centuries ago; it seems he has heard of the remaining Naloren and has sent his minion to investigate. You must not let it return to its master.",
		resolvers.nice_tile{image="invis.png", add_mos = {{image="npc/aquatic_demon_walrog.png"}}},
		level_range = {7, nil}, exp_worth = 2,
		rank = 4,
		instakill_immune = 1,
		max_life = 100, life_rating = 15, fixed_rating = true,
		autolevel = "caster",
		combat_armor = 15, combat_def = 0, combat_atk = 15,
		combat = {damtype=DamageType.ICE},
		body = { INVEN = 10, MAINHAND=1, OFFHAND=1, NECK=1, BODY=1 },
		resolvers.equip{
			{type="jewelry", subtype="amulet", defined = "PENDANT_DEEP", random_art_replace={chance=75}},
			{type="weapon", subtype="staff", autoreq=true, force_drop=true},
		},
		resolvers.drops{chance=100, nb=3, {tome_drops="boss"} },
		ai = "tactical",
		ai_tactic = resolvers.tactic"ranged",

		resists = {[DamageType.COLD] = 30, [DamageType.FIRE] = -30},
		resolvers.auto_equip_filters("Archmage"),
		auto_classes={{class="Archmage", start_level=12, level_rate=35}},

		-- Override the recalculated AI tactics to avoid problematic kiting in the early game
		low_level_tactics_override = {escape=0},

		on_die = function(self, who)
			--game.player:resolveSource():setQuestStatus("start-naloren", engine.Quest.COMPLETED, "abyssal-caves")
		end,

		resolvers.talents{ [Talents.T_TIDAL_WAVE]=2, [Talents.T_GLACIAL_VAPOUR]=2, [Talents.T_WATER_BOLT]=2 },

		--on_death_lore = "walrog",
	}
end

