-- ToME - Tales of Maj'Eyal
-- Copyright (C) 2009 - 2019 Nicolas Casalini
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Nicolas Casalini "DarkGod"
-- darkgod@te4.org

local Talents = require("engine.interface.ActorTalents")

newEntity{
	define_as = "BASE_NPC_NEW_SHELLSEA_TOWN",
	type = "humanoid", subtype = "nalore",
	display = "p", color=colors.WHITE,
	faction = "nalorenrace+nalore",

	combat = { dam=resolvers.rngavg(1,2), atk=2, apr=0, dammod={str=0.4} },

	body = { INVEN = 10, MAINHAND=1, OFFHAND=1,HEAD=1, BODY=1, QUIVER=1 },
	resolvers.drops{chance=20, nb=1, {} },
	lite = 1,

	life_rating = 10,
	rank = 2,
	size_category = 3,
	can_breath = {water = 1},
	open_door = true,

	autolevel = "warrior",
	ai = "dumb_talented_simple", ai_state = { ai_move="move_complex", talent_in=3, },
	stats = { str=20, dex=60, mag=20, con=60, cun = 60, wil = 20},
}

newEntity{ base = "BASE_NPC_NEW_SHELLSEA_TOWN",
	name = "naloren warrior", color=colors.YELLOW,
	desc = [[A warrior wielding a trident.]],
	level_range = {5, nil}, exp_worth = 1,
	rarity = 1,
	max_life = resolvers.rngavg(80,90),
	ai = "tactical",
	ai_tactic = resolvers.tactic"melee",
	resolvers.auto_equip_filters{MAINHAND = {subtype="trident"},},
	resolvers.equip{
		{type="weapon", subtype="trident", autoreq=true, force_drop=true, special_rarity="trident_rarity"},
		{type="armor", subtype="head", name = "iron helm", autoreq=true},
	},
	resolvers.talents{
		[Talents.T_ARMOUR_TRAINING]={base = 2, every = 6, max=8},
		[Talents.T_WEAPON_COMBAT]={base=1, every=10, max=6},
		[Talents.T_EXOTIC_WEAPONS_MASTERY]={base=1, every=10, max=6},
		[Talents.T_STUNNING_BLOW]={base = 2, every = 10, max=5},
		[Talents.T_RUSH]={base = 2, every = 10, max=5},
	},
	combat_armor = 20, combat_def = 20,
	stats = { str=60, dex=60, mag=20, con=60, cun = 20, wil = 20},
	resolvers.sustains_at_birth(),
}

newEntity{ base = "BASE_NPC_NEW_SHELLSEA_TOWN",
	name = "naloren guard", color=colors.LIGHT_UMBER,
	desc = [[A stern-looking guard, he will not let you disturb the town.]],
	level_range = {1, nil}, exp_worth = 1,
	rarity = 3,
	max_life = resolvers.rngavg(70,80),
	resolvers.equip{
		{type="weapon", subtype="longsword", autoreq=true},
		{type="armor", subtype="shield", autoreq=true},
	},
	combat_armor = 20, combat_def = 20,
	stats = { str=60, dex=60, mag=20, con=60, cun = 20, wil = 20},
	resolvers.talents{
		[Talents.T_WEAPON_COMBAT]={base=1, every=10, max=6},
		[Talents.T_WEAPONS_MASTERY]={base=1, every=10, max=6},
		[Talents.T_ARMOUR_TRAINING]={base=2, every=10, max=6},
		[Talents.T_RUSH]={start = 5, base=1, every = 10, max=5},
		[Talents.T_PERFECT_STRIKE]={start = 5, base=1, every = 10, max=5},
	},
}

newEntity{ base = "BASE_NPC_NEW_SHELLSEA_TOWN",
	name = "naloren archer", color=colors.UMBER,
	desc = [[A stern-looking guard, he will not let you disturb the town.]],
	level_range = {1, nil}, exp_worth = 1,
	rarity = 3,
	max_life = resolvers.rngavg(50,60),
	resolvers.talents{
		[Talents.T_WEAPON_COMBAT]={base=1, every=10, max=6},
		[Talents.T_BOW_MASTERY]={base=1, every=10, max=6},
		[Talents.T_SHOOT]=1,
	},
	ai_state = { talent_in=2, },
	autolevel = "archer",
	stats = { str=60, dex=60, mag=20, con=60, cun = 20, wil = 20},
	resolvers.equip{ {type="weapon", subtype="longbow", autoreq=true}, {type="ammo", subtype="arrow", autoreq=true} },
}

newEntity{ base = "BASE_NPC_NEW_SHELLSEA_TOWN",
	name = "naloren water mage", color=colors.BLUE,
	desc = [[An elven water mage.]],
	level_range = {3, nil}, exp_worth = 1,
	rarity = 1,
	ai = "tactical",
	ai_tactic = resolvers.tactic"ranged",
	max_life = resolvers.rngavg(70,80),
	resolvers.equip{
		{type="weapon", subtype="staff", forbid_power_source={antimagic=true}, autoreq=true},
		{type="armor", subtype="cloth", forbid_power_source={antimagic=true}, autoreq=true},
	},
	resolvers.talents{
		[Talents.T_STAFF_MASTERY]={base=1, every=10, max=5},
		[Talents.T_WATER_BOLT]={base = 2, every = 7, max = 7},
		[Talents.T_ICE_SHARDS]={base = 3, every = 8, max = 7},
		[Talents.T_FREEZE]={base = 2, every = 7, max = 7},
	},
	autolevel = "caster",
	stats = { str=20, dex=20, mag=60, con=20, cun = 60, wil = 60},
	resolvers.sustains_at_birth(),
}
